import time
import json
try:
  from urllib import urlencode
  from urlparse import urljoin
except ImportError:
  from urllib.parse import urlencode
  from urllib.parse import urljoin

import hmac
import hashlib
try:
  from Crypto.Cipher import AES
except ImportError:
  encrypted = False
else:
  import getpass
  import ast
  import json
  encrypted = True

import requests


BITTREX_BASE_URL = 'https://bittrex.com/api/v1.1/{method_set}/{method}?' # see https://bittrex.com/home/api
BITTREX_AUTHENTICATED_MARKET_METHODS = {
  'getopenorders',
  'cancel',
  'sellmarket',
  'selllimit',
  'buymarket',
  'buylimit'
}
BITTREX_AUTHENTICATED_ACCOUNT_METHODS = {
  'getbalances',
  'getbalance',
  'getdepositaddress',
  'withdraw',
  'getorderhistory',
  'getorder',
  'getdeposithistory',
  'getwithdrawalhistory'
}


def using_requests(request_url, apisign=None):
  if apisign:
    return  requests.get(
              request_url,
              headers={'apisign': apisign}
            ).json()
  else:
    return  requests.get(
              request_url
            ).json()


class Bittrex(object):
  '''
  Bittrex API wrapper
  '''
  def __init__(self, api_key=None, api_secret=None, calls_per_second=1, dispatch=using_requests):
    self.api_key = str(api_key) if api_key is not None else ''
    self.api_secret = str(api_secret) if api_secret is not None else ''
    self.dispatch = dispatch
    self.call_rate = 1.0/calls_per_second
    self.last_call = None

  def decrypt(self):
    if encrypted:
      cipher = AES.new(getpass.getpass(
        'Input decryption password (string will not show)'))
      try:
        if isinstance(self.api_key, str):
          self.api_key = ast.literal_eval(self.api_key)
        if isinstance(self.api_secret, str):
          self.api_secret = ast.literal_eval(self.api_secret)
      except Exception:
        pass
      self.api_key = cipher.decrypt(self.api_key).decode()
      self.api_secret = cipher.decrypt(self.api_secret).decode()
    else:
      raise ImportError('pycrypto module has to be installed')

  def wait(self):
    if self.last_call is None:
      self.last_call = time.time()
    else:
      now = time.time()
      passed = now - self.last_call
      if passed < self.call_rate:
        #print('sleep')
        time.sleep(1.0 - passed)

      self.last_call = time.time()

  def api_query(self, method, options=None):
    '''
    Queries Bittrex with given method and options.

    :param method: Query method for getting info
    :type method: str
    :param options: Extra options for query
    :type options: dict
    :return: JSON response from Bittrex
    :rtype : dict
    '''
    if not options:
      options = {}
    nonce = str(int(time.time() * 1000))
    method_set = 'public'

    if method in BITTREX_AUTHENTICATED_MARKET_METHODS:
      method_set = 'market'
    elif method in BITTREX_AUTHENTICATED_ACCOUNT_METHODS:
      method_set = 'account'

    request_url = BITTREX_BASE_URL.format(method_set=method_set, method=method)

    if method_set != 'public':
      request_url = '{0}apikey={1}&nonce={2}&'.format(
        request_url, self.api_key, nonce)

    request_url += urlencode(options)

    apisign = hmac.new(self.api_secret.encode(),
                       request_url.encode(),
                       hashlib.sha512).hexdigest()

    self.wait()

    return self.dispatch(request_url, apisign)

  def get_markets(self):
    '''
    Used to get the open and available trading markets
    at Bittrex along with other meta data.

    Endpoint: /public/getmarkets

    Example ::
      {'success': True,
       'message': '',
       'result': [ {'MarketCurrency': 'LTC',
              'BaseCurrency': 'BTC',
              'MarketCurrencyLong': 'Litecoin',
              'BaseCurrencyLong': 'Bitcoin',
              'MinTradeSize': 1e-08,
              'MarketName': 'BTC-LTC',
              'IsActive': True,
              'Created': '2014-02-13T00:00:00',
              'Notice': None,
              'IsSponsored': None,
              'LogoUrl': 'https://i.imgur.com/R29q3dD.png'},
              ...
            ]
      }

    :return: Available market info in JSON
    :rtype : dict
    '''
    return self.api_query('getmarkets')

  def get_currencies(self):
    '''
    Used to get all supported currencies at Bittrex
    along with other meta data.

    Endpoint: /public/getcurrencies

    :return: Supported currencies info in JSON
    :rtype : dict
    '''
    return self.api_query('getcurrencies')

  def get_ticker(self, market):
    '''
    Used to get the current tick values for a market.

    Endpoint: /public/getticker

    :param market: String literal for the market (ex: BTC-LTC)
    :type market: str
    :return: Current values for given market in JSON
    :rtype : dict
    '''
    return self.api_query('getticker', {'market': market})

  def get_market_summaries(self):
    '''
    Used to get the last 24 hour summary of all active exchanges

    Endpoint: /public/getmarketsummary

    :return: Summaries of active exchanges in JSON
    :rtype : dict
    '''
    return self.api_query('getmarketsummaries')

  def get_marketsummary(self, market):
    '''
    Used to get the last 24 hour summary of all active
    exchanges in specific coin

    Endpoint: /public/getmarketsummary

    :param market: String literal for the market(ex: BTC-XRP)
    :type market: str
    :return: Summaries of active exchanges of a coin in JSON
    :rtype : dict
    '''
    return self.api_query('getmarketsummary', {'market': market})

  def get_orderbook(self, market, depth_type, depth=20):
    '''
    Used to get retrieve the orderbook for a given market

    Endpoint: /public/getorderbook

    :param market: String literal for the market (ex: BTC-LTC)
    :type market: str
    :param depth_type: buy, sell or both to identify the type of
      orderbook to return.
      Use constants BUY_ORDERBOOK, SELL_ORDERBOOK, BOTH_ORDERBOOK
    :type depth_type: str
    :param depth: how deep of an order book to retrieve.
      Max is 100, default is 20
    :type depth: int
    :return: Orderbook of market in JSON
    :rtype : dict
    '''
    return self.api_query('getorderbook',
                          {'market': market,
                           'type': depth_type,
                           'depth': depth})

  def get_market_history(self, market, count=20):
    '''
    Used to retrieve the latest trades that have occurred for a
    specific market.

    Endpoint: /market/getmarkethistory

    Example ::
      {'success': True,
      'message': '',
      'result': [ {'Id': 5625015,
             'TimeStamp': '2017-08-31T01:29:50.427',
             'Quantity': 7.31008193,
             'Price': 0.00177639,
             'Total': 0.01298555,
             'FillType': 'FILL',
             'OrderType': 'BUY'},
             ...
             ]
      }

    :param market: String literal for the market (ex: BTC-LTC)
    :type market: str
    :param count: Number between 1-100 for the number
      of entries to return (default = 20)
    :type count: int
    :return: Market history in JSON
    :rtype : dict
    '''
    return self.api_query('getmarkethistory',
                          {'market': market, 'count': count})

  def buy_limit(self, market, quantity, rate):
    '''
    Used to place a buy order in a specific market. Use buylimit to place
    limit orders Make sure you have the proper permissions set on your
    API keys for this call to work

    Endpoint: /market/buylimit

    :param market: String literal for the market (ex: BTC-LTC)
    :type market: str
    :param quantity: The amount to purchase
    :type quantity: float
    :param rate: The rate at which to place the order.
      This is not needed for market orders
    :type rate: float
    :return:
    :rtype : dict
    '''
    return self.api_query('buylimit',
                          {'market': market,
                           'quantity': quantity,
                           'rate': rate})

  def sell_limit(self, market, quantity, rate):
    '''
    Used to place a sell order in a specific market. Use selllimit to place
    limit orders Make sure you have the proper permissions set on your
    API keys for this call to work

    Endpoint: /market/selllimit

    :param market: String literal for the market (ex: BTC-LTC)
    :type market: str
    :param quantity: The amount to purchase
    :type quantity: float
    :param rate: The rate at which to place the order.
      This is not needed for market orders
    :type rate: float
    :return:
    :rtype : dict
    '''
    return self.api_query('selllimit',
                          {'market': market,
                           'quantity': quantity,
                           'rate': rate})

  def cancel(self, uuid):
    '''
    Used to cancel a buy or sell order

    Endpoint: /market/cancel

    :param uuid: uuid of buy or sell order
    :type uuid: str
    :return:
    :rtype : dict
    '''
    return self.api_query('cancel', {'uuid': uuid})

  def get_open_orders(self, market=None):
    '''
    Get all orders that you currently have opened.
    A specific market can be requested.

    Endpoint: /market/getopenorders

    :param market: String literal for the market (ie. BTC-LTC)
    :type market: str
    :return: Open orders info in JSON
    :rtype : dict
    '''
    return self.api_query('getopenorders',
                          {'market': market} if market else None)

  def get_balances(self):
    '''
    Used to retrieve all balances from your account.

    Endpoint: /account/getbalances

    Example ::
      {'success': True,
       'message': '',
       'result': [ {'Currency': '1ST',
              'Balance': 10.0,
              'Available': 10.0,
              'Pending': 0.0,
              'CryptoAddress': None},
              ...
            ]
      }


    :return: Balances info in JSON
    :rtype : dict
    '''
    return self.api_query('getbalances', {})

  def get_balance(self, currency):
    '''
    Used to retrieve the balance from your account for a specific currency

    Endpoint: /account/getbalance

    Example ::
      {'success': True,
       'message': '',
       'result': {'Currency': '1ST',
            'Balance': 10.0,
            'Available': 10.0,
            'Pending': 0.0,
            'CryptoAddress': None}
      }


    :param currency: String literal for the currency (ex: LTC)
    :type currency: str
    :return: Balance info in JSON
    :rtype : dict
    '''
    return self.api_query('getbalance', {'currency': currency})

  def get_deposit_address(self, currency):
    '''
    Used to generate or retrieve an address for a specific currency

    Endpoint: /account/getdepositaddress

    :param currency: String literal for the currency (ie. BTC)
    :type currency: str
    :return: Address info in JSON
    :rtype : dict
    '''
    return self.api_query('getdepositaddress', {'currency': currency})

  def withdraw(self, currency, quantity, address):
    '''
    Used to withdraw funds from your account

    Endpoint: /account/withdraw

    :param currency: String literal for the currency (ie. BTC)
    :type currency: str
    :param quantity: The quantity of coins to withdraw
    :type quantity: float
    :param address: The address where to send the funds.
    :type address: str
    :return:
    :rtype : dict
    '''
    return self.api_query('withdraw',
                          {'currency': currency,
                           'quantity': quantity,
                           'address': address})

  def get_order_history(self, market=None):
    '''
    Used to retrieve order trade history of account

    Endpoint: /account/getorderhistory

    :param market: optional a string literal for the market (ie. BTC-LTC).
      If omitted, will return for all markets
    :type market: str
    :return: order history in JSON
    :rtype : dict
    '''
    return self.api_query('getorderhistory',
                          {'market': market} if market else None)

  def get_order(self, uuid):
    '''
    Used to get details of buy or sell order

    Endpoint: /account/getorder

    :param uuid: uuid of buy or sell order
    :type uuid: str
    :return:
    :rtype : dict
    '''
    return self.api_query('getorder', {'uuid': uuid})

  def get_withdrawal_history(self, currency=None):
    '''
    Used to view your history of withdrawals

    Endpoint: /account/getwithdrawalhistory

    :param currency: String literal for the currency (ie. BTC)
    :type currency: str
    :return: withdrawal history in JSON
    :rtype : dict
    '''

    return self.api_query('getwithdrawalhistory',
                          {'currency': currency} if currency else None)

  def get_deposit_history(self, currency=None):
    '''
    Used to view your history of deposits

    Endpoint: /account/getdeposithistory

    :param currency: String literal for the currency (ie. BTC)
    :type currency: str
    :return: deposit history in JSON
    :rtype : dict
    '''
    return self.api_query('getdeposithistory',
                          {'currency': currency} if currency else None)

  def list_markets_by_currency(self, currency):
    '''
    Helper function to see which markets exist for a currency.

    Endpoint: /public/getmarkets

    Example ::
      >>> Bittrex(None, None).list_markets_by_currency('LTC')
      ['BTC-LTC', 'ETH-LTC', 'USDT-LTC']

    :param currency: String literal for the currency (ex: LTC)
    :type currency: str
    :return: List of markets that the currency appears in
    :rtype: list
    '''
    return [market['MarketName'] for market in self.get_markets()['result']
      if market['MarketName'].lower().endswith(currency.lower())]
